@@ -270,125 +270,17 @@ struct QSSGShaderKeyTextureChannel : public QSSGShaderKeyUnsigned<2>
     }
 };
 
-struct QSSGShaderKeyTextureSwizzle : public QSSGShaderKeyUnsigned<5>
-{
-    enum TextureSwizzleBits {
-        noSwizzle = 1 << 0,
-        L8toR8 = 1 << 1,
-        A8toR8 = 1 << 2,
-        L8A8toRG8 = 1 << 3,
-        L16toR16 = 1 << 4
-    };
-
-    explicit QSSGShaderKeyTextureSwizzle(const char *inName = "") : QSSGShaderKeyUnsigned<5>(inName) {}
-
-    bool getBitValue(TextureSwizzleBits swizzleBit, QSSGDataView<quint32> inKeySet) const
-    {
-        return (getValue(inKeySet) & swizzleBit) ? true : false;
-    }
-
-    void setBitValue(TextureSwizzleBits swizzleBit, bool inValue, QSSGDataRef<quint32> inKeySet)
-    {
-        quint32 theValue = getValue(inKeySet);
-        quint32 mask = swizzleBit;
-        if (inValue) {
-            theValue = theValue | mask;
-        } else {
-            mask = ~mask;
-            theValue = theValue & mask;
-        }
-        setValue(inKeySet, theValue);
-    }
-
-    void setSwizzleMode(QSSGDataRef<quint32> inKeySet, QSSGRenderTextureSwizzleMode swizzleMode, bool val)
-    {
-        switch (swizzleMode) {
-        case QSSGRenderTextureSwizzleMode::NoSwizzle:
-            setBitValue(noSwizzle, val, inKeySet);
-            break;
-        case QSSGRenderTextureSwizzleMode::L8toR8:
-            setBitValue(L8toR8, val, inKeySet);
-            break;
-        case QSSGRenderTextureSwizzleMode::A8toR8:
-            setBitValue(A8toR8, val, inKeySet);
-            break;
-        case QSSGRenderTextureSwizzleMode::L8A8toRG8:
-            setBitValue(L8A8toRG8, val, inKeySet);
-            break;
-        case QSSGRenderTextureSwizzleMode::L16toR16:
-            setBitValue(L16toR16, val, inKeySet);
-            break;
-        }
-    }
-
-    bool isNoSwizzled(QSSGDataView<quint32> inKeySet) const { return getBitValue(noSwizzle, inKeySet); }
-    void setNoSwizzled(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(noSwizzle, val, inKeySet); }
-
-    bool isL8Swizzled(QSSGDataView<quint32> inKeySet) const { return getBitValue(L8toR8, inKeySet); }
-    void setL8Swizzled(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(L8toR8, val, inKeySet); }
-
-    bool isA8Swizzled(QSSGDataView<quint32> inKeySet) const { return getBitValue(A8toR8, inKeySet); }
-    void setA8Swizzled(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(A8toR8, val, inKeySet); }
-
-    bool isL8A8Swizzled(QSSGDataView<quint32> inKeySet) const { return getBitValue(L8A8toRG8, inKeySet); }
-    void setL8A8Swizzled(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(L8A8toRG8, val, inKeySet); }
-
-    bool isL16Swizzled(QSSGDataView<quint32> inKeySet) const { return getBitValue(L16toR16, inKeySet); }
-    void setL16Swizzled(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(L16toR16, val, inKeySet); }
-
-    void toString(QByteArray &ioStr, QSSGDataView<quint32> inKeySet) const
-    {
-        ioStr.append(name);
-        ioStr.append(QByteArrayView("={"));
-        internalToString(ioStr, QByteArrayView("noswizzle"), isNoSwizzled(inKeySet));
-        ioStr.append(';');
-        internalToString(ioStr, QByteArrayView("l8swizzle"), isL8Swizzled(inKeySet));
-        ioStr.append(';');
-        internalToString(ioStr, QByteArrayView("a8swizzle"), isA8Swizzled(inKeySet));
-        ioStr.append(';');
-        internalToString(ioStr, QByteArrayView("l8a8swizzle"), isL8A8Swizzled(inKeySet));
-        ioStr.append(';');
-        internalToString(ioStr, QByteArrayView("l16swizzle"), isL16Swizzled(inKeySet));
-        ioStr.append('}');
-    }
-    void fromString(const QByteArray &ioStr, QSSGDataRef<quint32> inKeySet)
-    {
-        const qsizetype nameLen = name.size();
-        const qsizetype strOffset = ioStr.indexOf(name);
-        if (strOffset >= 0) {
-            /* The key is stored as name={;;;;;} */
-            if (ioStr[strOffset + nameLen] != '=')
-                return;
-            if (ioStr[strOffset + nameLen + 1] != '{')
-                return;
-            const int codeOffsetBegin = strOffset + nameLen + 2;
-            int codeOffset = 0;
-            while (ioStr[codeOffsetBegin + codeOffset] != '}')
-                codeOffset++;
-            const QList<QByteArray> list = ioStr.mid(codeOffsetBegin, codeOffset).split(';');
-            if (list.size() != 5)
-                return;
-            setNoSwizzled(inKeySet, getBoolValue(list[0], QByteArrayView("noswizzle")));
-            setL8Swizzled(inKeySet, getBoolValue(list[1], QByteArrayView("l8swizzle")));
-            setA8Swizzled(inKeySet, getBoolValue(list[2],  QByteArrayView("a8swizzle")));
-            setL8A8Swizzled(inKeySet, getBoolValue(list[3], QByteArrayView("l8a8swizzle")));
-            setL16Swizzled(inKeySet, getBoolValue(list[4], QByteArrayView("l16swizzle")));
-        }
-    }
-};
-
-struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<6>
+struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<5>
 {
     enum ImageMapBits {
         Enabled = 1 << 0,
         EnvMap = 1 << 1,
         LightProbe = 1 << 2,
-        InvertUV = 1 << 3,
-        Premultiplied = 1 << 4,
-        Identity = 1 << 5
+        Identity = 1 << 3,
+        UsesUV1 = 1 << 4
     };
 
-    explicit QSSGShaderKeyImageMap(const char *inName = "") : QSSGShaderKeyUnsigned<6>(inName) {}
+    explicit QSSGShaderKeyImageMap(const char *inName = "") : QSSGShaderKeyUnsigned<5>(inName) {}
 
     bool getBitValue(ImageMapBits imageBit, QSSGDataView<quint32> inKeySet) const
     {
@@ -417,15 +309,12 @@ struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<6>
     bool isLightProbe(QSSGDataView<quint32> inKeySet) const { return getBitValue(LightProbe, inKeySet); }
     void setLightProbe(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(LightProbe, val, inKeySet); }
 
-    bool isInvertUVMap(QSSGDataView<quint32> inKeySet) const { return getBitValue(InvertUV, inKeySet); }
-    void setInvertUVMap(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(InvertUV, val, inKeySet); }
-
-    bool isPremultiplied(QSSGDataView<quint32> inKeySet) const { return getBitValue(Premultiplied, inKeySet); }
-    void setPremultiplied(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(Premultiplied, val, inKeySet); }
-
     bool isIdentityTransform(QSSGDataView<quint32> inKeySet) const { return getBitValue(Identity, inKeySet); }
     void setIdentityTransform(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(Identity, val, inKeySet); }
 
+    bool isUsingUV1(QSSGDataView<quint32> inKeySet) const { return getBitValue(UsesUV1, inKeySet); }
+    void setUsesUV1(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(UsesUV1, val, inKeySet); }
+
     void toString(QByteArray &ioStr, QSSGDataView<quint32> inKeySet) const
     {
         ioStr.append(name);
@@ -436,11 +325,9 @@ struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<6>
         ioStr.append(';');
         internalToString(ioStr, QByteArrayView("lightProbe"), isLightProbe(inKeySet));
         ioStr.append(';');
-        internalToString(ioStr, QByteArrayView("invertUV"), isInvertUVMap(inKeySet));
-        ioStr.append(';');
-        internalToString(ioStr, QByteArrayView("premultiplied"), isPremultiplied(inKeySet));
-        ioStr.append(';');
         internalToString(ioStr, QByteArrayView("identity"), isIdentityTransform(inKeySet));
+        ioStr.append(';');
+        internalToString(ioStr, QByteArrayView("usesUV1"), isUsingUV1(inKeySet));
         ioStr.append('}');
     }
 };
@@ -635,7 +522,7 @@ struct QSSGShaderDefaultMaterialKeyProperties
         LightCount = QSSG_MAX_NUM_LIGHTS,
     };
     enum {
-        SingleChannelImageCount = 5,
+        SingleChannelImageCount = 10,
     };
     enum ImageMapNames {
         DiffuseMap = 0,
@@ -645,15 +532,18 @@ struct QSSGShaderDefaultMaterialKeyProperties
         BumpMap,
         SpecularAmountMap,
         NormalMap,
-        LightmapIndirect,
-        LightmapRadiosity,
-        LightmapShadow,
+        ClearcoatNormalMap,
         // single channel images
         OpacityMap,
         RoughnessMap,
         MetalnessMap,
         OcclusionMap,
         TranslucencyMap,
+        HeightMap,
+        ClearcoatMap,
+        ClearcoatRoughnessMap,
+        TransmissionMap,
+        ThicknessMap,
 
         ImageMapCount,
         SingleChannelImagesFirst = OpacityMap
@@ -664,6 +554,14 @@ struct QSSGShaderDefaultMaterialKeyProperties
         MetalnessChannel,
         OcclusionChannel,
         TranslucencyChannel,
+        HeightChannel,
+        ClearcoatChannel,
+        ClearcoatRoughnessChannel,
+        TransmissionChannel,
+        ThicknessChannel
+    };
+    enum {
+        MorphTargetCount = 8,
     };
 
     QSSGShaderKeyBoolean m_hasLighting;
@@ -678,7 +576,6 @@ struct QSSGShaderDefaultMaterialKeyProperties
     QSSGShaderKeyBoolean m_vertexColorsEnabled;
     QSSGShaderKeySpecularModel m_specularModel;
     QSSGShaderKeyImageMap m_imageMaps[ImageMapCount];
-    QSSGShaderKeyTextureSwizzle m_textureSwizzle[ImageMapCount];
     QSSGShaderKeyTextureChannel m_textureChannels[SingleChannelImageCount];
     QSSGShaderKeyUnsigned<16> m_boneCount;
     QSSGShaderKeyBoolean m_isDoubleSided;
@@ -691,6 +588,12 @@ struct QSSGShaderDefaultMaterialKeyProperties
     QSSGShaderKeyVertexAttribute m_vertexAttributes;
     QSSGShaderKeyBoolean m_usesFloatJointIndices;
     qsizetype m_stringBufferSizeHint = 0;
+    QSSGShaderKeyBoolean m_usesInstancing;
+    QSSGShaderKeyUnsigned<4> m_morphTargetCount;
+    QSSGShaderKeyVertexAttribute m_morphTargetAttributes[MorphTargetCount];
+    QSSGShaderKeyBoolean m_blendParticles;
+    QSSGShaderKeyBoolean m_clearcoatEnabled;
+    QSSGShaderKeyBoolean m_transmissionEnabled;
 
     QSSGShaderDefaultMaterialKeyProperties()
         : m_hasLighting("hasLighting")
@@ -710,6 +613,11 @@ struct QSSGShaderDefaultMaterialKeyProperties
         , m_alphaMode("alphaMode")
         , m_vertexAttributes("vertexAttributes")
         , m_usesFloatJointIndices("usesFloatJointIndices")
+        , m_usesInstancing("usesInstancing")
+        , m_morphTargetCount("morphTargetCount")
+        , m_blendParticles("blendParticles")
+        , m_clearcoatEnabled("clearcoatEnabled")
+        , m_transmissionEnabled("transmissionEnabled")
     {
         m_lightFlags[0].name = "light0HasPosition";
         m_lightFlags[1].name = "light1HasPosition";
@@ -782,36 +690,37 @@ struct QSSGShaderDefaultMaterialKeyProperties
         m_imageMaps[4].name = "bumpMap";
         m_imageMaps[5].name = "specularAmountMap";
         m_imageMaps[6].name = "normalMap";
-        m_imageMaps[7].name = "lightmapIndirect";
-        m_imageMaps[8].name = "lightmapRadiosity";
-        m_imageMaps[9].name = "lightmapShadow";
-        m_imageMaps[10].name = "opacityMap";
-        m_imageMaps[11].name = "roughnessMap";
-        m_imageMaps[12].name = "metalnessMap";
-        m_imageMaps[13].name = "occlusionMap";
-        m_imageMaps[14].name = "translucencyMap";
-
-        m_textureSwizzle[0].name = "diffuseMap_swizzle";
-        m_textureSwizzle[1].name = "emissiveMap_swizzle";
-        m_textureSwizzle[2].name = "specularMap_swizzle";
-        m_textureSwizzle[3].name = "baseColorMap_swizzle";
-        m_textureSwizzle[4].name = "bumpMap_swizzle";
-        m_textureSwizzle[5].name = "specularAmountMap_swizzle";
-        m_textureSwizzle[6].name = "normalMap_swizzle";
-        m_textureSwizzle[7].name = "lightmapIndirect_swizzle";
-        m_textureSwizzle[8].name = "lightmapRadiosity_swizzle";
-        m_textureSwizzle[9].name = "lightmapShadow_swizzle";
-        m_textureSwizzle[10].name = "opacityMap_swizzle";
-        m_textureSwizzle[11].name = "roughnessMap_swizzle";
-        m_textureSwizzle[12].name = "metalnessMap_swizzle";
-        m_textureSwizzle[13].name = "occlusionMap_swizzle";
-        m_textureSwizzle[14].name = "translucencyMap_swizzle";
+        m_imageMaps[7].name = "clearcoatNormalMap";
+        m_imageMaps[8].name = "opacityMap";
+        m_imageMaps[9].name = "roughnessMap";
+        m_imageMaps[10].name = "metalnessMap";
+        m_imageMaps[11].name = "occlusionMap";
+        m_imageMaps[12].name = "translucencyMap";
+        m_imageMaps[13].name = "heightMap";
+        m_imageMaps[14].name = "clearcoatMap";
+        m_imageMaps[15].name = "clearcoatRoughnessMap";
+        m_imageMaps[16].name = "transmissionMap";
+        m_imageMaps[17].name = "thicknessMap";
 
         m_textureChannels[0].name = "opacityMap_channel";
         m_textureChannels[1].name = "roughnessMap_channel";
         m_textureChannels[2].name = "metalnessMap_channel";
         m_textureChannels[3].name = "occlusionMap_channel";
         m_textureChannels[4].name = "translucencyMap_channel";
+        m_textureChannels[5].name = "heightMap_channel";
+        m_textureChannels[6].name = "clearcoatMap_channel";
+        m_textureChannels[7].name = "clearcoatRoughnessMap_channel";
+        m_textureChannels[8].name = "transmissionMap_channel";
+        m_textureChannels[9].name = "thicknessMap_channel";
+
+        m_morphTargetAttributes[0].name = "morphTarget0Attributes";
+        m_morphTargetAttributes[1].name = "morphTarget1Attributes";
+        m_morphTargetAttributes[2].name = "morphTarget2Attributes";
+        m_morphTargetAttributes[3].name = "morphTarget3Attributes";
+        m_morphTargetAttributes[4].name = "morphTarget4Attributes";
+        m_morphTargetAttributes[5].name = "morphTarget5Attributes";
+        m_morphTargetAttributes[6].name = "morphTarget6Attributes";
+        m_morphTargetAttributes[7].name = "morphTarget7Attributes";
 
         init();
     }
@@ -840,10 +749,8 @@ struct QSSGShaderDefaultMaterialKeyProperties
         inVisitor.visit(m_vertexColorsEnabled);
         inVisitor.visit(m_specularModel);
 
-        for (quint32 idx = 0, end = ImageMapCount; idx < end; ++idx) {
+        for (quint32 idx = 0, end = ImageMapCount; idx < end; ++idx)
             inVisitor.visit(m_imageMaps[idx]);
-            inVisitor.visit(m_textureSwizzle[idx]);
-        }
 
         for (auto &textureChannel : m_textureChannels)
             inVisitor.visit(textureChannel);
@@ -858,6 +765,13 @@ struct QSSGShaderDefaultMaterialKeyProperties
         inVisitor.visit(m_alphaMode);
         inVisitor.visit(m_vertexAttributes);
         inVisitor.visit(m_usesFloatJointIndices);
+        inVisitor.visit(m_usesInstancing);
+        inVisitor.visit(m_morphTargetCount);
+        for (quint32 idx = 0, end = MorphTargetCount; idx < end; ++idx)
+            inVisitor.visit(m_morphTargetAttributes[idx]);
+        inVisitor.visit(m_blendParticles);
+        inVisitor.visit(m_clearcoatEnabled);
+        inVisitor.visit(m_transmissionEnabled);
     }
 
     struct OffsetVisitor
@@ -872,7 +786,7 @@ struct QSSGShaderDefaultMaterialKeyProperties
             // This cost a few extra bits but prevents tedious errors like
             // loosing shader key bits because they got moved beyond the 32 border
             quint32 bit = m_offset % 32;
-            if (bit + TPropType::BitWidth > 31) {
+            if (bit + TPropType::BitWidth > 32) {
                 m_offset += 32 - bit;
             }
 
@@ -910,7 +824,7 @@ struct QSSGShaderDefaultMaterialKeyProperties
         visitProperties(visitor);
 
         // If this assert fires, then the default material key needs more bits.
-        Q_ASSERT(visitor.offsetVisitor.m_offset < 320);
+        Q_ASSERT(visitor.offsetVisitor.m_offset < 384);
         // This is so we can do some guestimate of how big the string buffer needs
         // to be to avoid doing a lot of allocations when concatenating the strings.
         m_stringBufferSizeHint = visitor.stringSizeVisitor.size;
@@ -920,7 +834,7 @@ struct QSSGShaderDefaultMaterialKeyProperties
 struct QSSGShaderDefaultMaterialKey
 {
     enum {
-        DataBufferSize = 10,
+        DataBufferSize = 12,
     };
     quint32 m_dataBuffer[DataBufferSize];
     size_t m_featureSetHash;
